@@ -15,15 +15,17 @@ describe "File.mtime" do
     File.mtime(@filename).should be_close(@mtime, TIME_TOLERANCE)
   end
 
-  guard -> { platform_is :linux or (platform_is :windows and ruby_version_is '2.5') } do
-    it "returns the modification Time of the file with microseconds" do
-      supports_subseconds = Integer(`stat -c%y '#{__FILE__}'`[/\.(\d+)/, 1], 10)
-      if supports_subseconds != 0
-        expected_time = Time.at(Time.now.to_i + 0.123456)
-        File.utime 0, expected_time, @filename
-        File.mtime(@filename).usec.should == expected_time.usec
-      else
-        File.mtime(__FILE__).usec.should == 0
+  platform_is :linux, :windows do
+    unless ENV.key?('TRAVIS') # https://bugs.ruby-lang.org/issues/17926
+      it "returns the modification Time of the file with microseconds" do
+        supports_subseconds = Integer(`stat -c%y '#{__FILE__}'`[/\.(\d{1,6})/, 1], 10)
+        if supports_subseconds != 0
+          expected_time = Time.at(Time.now.to_i + 0.123456)
+          File.utime 0, expected_time, @filename
+          File.mtime(@filename).usec.should == expected_time.usec
+        else
+          File.mtime(__FILE__).usec.should == 0
+        end
       end
     end
   end
