@@ -6,17 +6,21 @@ rescue LoadError
   return
 end
 
-class TestOptionParser::DidYouMean < TestOptionParser
+class TestOptionParserDidYouMean < TestOptionParser
   def setup
     super
     @opt.def_option("--foo", Integer) { |v| @foo = v }
     @opt.def_option("--bar", Integer) { |v| @bar = v }
     @opt.def_option("--baz", Integer) { |v| @baz = v }
     @formatter = ::DidYouMean.formatter
-    case @formatter
-    when ::DidYouMean::PlainFormatter
+    if ::DidYouMean.const_defined?(:Formatter)
+      ::DidYouMean.formatter = ::DidYouMean::Formatter
     else
-      ::DidYouMean.formatter = ::DidYouMean::PlainFormatter.new
+      case @formatter
+      when ::DidYouMean::PlainFormatter
+      else
+        ::DidYouMean.formatter = ::DidYouMean::PlainFormatter.new
+      end
     end
   end
 
@@ -36,15 +40,7 @@ class TestOptionParser::DidYouMean < TestOptionParser
     end
   end
 
-  def test_verbose
-    require 'did_you_mean/formatters/verbose_formatter'
-    ::DidYouMean.formatter = ::DidYouMean::VerboseFormatter.new
-    assert_raise_with_message(OptionParser::InvalidOption, /invalid option: --baa\n\s+Did you mean\?\s+bar\s+baz\s*\Z/) do
-      @opt.permute!(%w"--baa")
-    end
-  end
-
-  def test_ambiguos
+  def test_ambiguous
     assert_raise_with_message(OptionParser::AmbiguousOption, /ambiguous option: --ba\nDid you mean\?\s+bar\s+baz\Z/) do
       @opt.permute!(%w"--ba")
     end
