@@ -47,12 +47,19 @@ class TestCall < Test::Unit::TestCase
     assert_equal(5, o.y)
     o&.z ||= 6
     assert_equal(6, o.z)
+    o&.z &&= 7
+    assert_equal(7, o.z)
 
     o = nil
     assert_nil(o&.x)
     assert_nothing_raised(NoMethodError) {o&.x = raise}
+    assert_nothing_raised(NoMethodError) {o&.x = raise; nil}
     assert_nothing_raised(NoMethodError) {o&.x *= raise}
     assert_nothing_raised(NoMethodError) {o&.x *= raise; nil}
+    assert_nothing_raised(NoMethodError) {o&.x ||= raise}
+    assert_nothing_raised(NoMethodError) {o&.x ||= raise; nil}
+    assert_nothing_raised(NoMethodError) {o&.x &&= raise}
+    assert_nothing_raised(NoMethodError) {o&.x &&= raise; nil}
   end
 
   def test_safe_call_evaluate_arguments_only_method_call_is_made
@@ -99,4 +106,13 @@ class TestCall < Test::Unit::TestCase
     ary = [1, 2]
     assert_equal([0, 1, 2, 1], aaa(0, *ary, ary.shift), bug12860)
   end
+
+  def test_call_block_order
+    bug16504 = '[ruby-core:96769] [Bug# 16504]'
+    b = proc{}
+    ary = [1, 2, b]
+    assert_equal([1, 2, b], aaa(*ary, &ary.pop), bug16504)
+    ary = [1, 2, b]
+    assert_equal([0, 1, 2, b], aaa(0, *ary, &ary.pop), bug16504)
+  end
 end
