@@ -1,8 +1,9 @@
-require 'rubygems/test_case'
-require 'rubygems/name_tuple'
+# frozen_string_literal: true
+
+require_relative "helper"
+require "rubygems/name_tuple"
 
 class TestGemNameTuple < Gem::TestCase
-
   def test_full_name
     n = Gem::NameTuple.new "a", Gem::Version.new(0), "ruby"
     assert_equal "a-0", n.full_name
@@ -34,11 +35,9 @@ class TestGemNameTuple < Gem::TestCase
   end
 
   def test_spaceship
-    a   = Gem::NameTuple.new 'a', Gem::Version.new(0), Gem::Platform::RUBY
-    a_p = Gem::NameTuple.new 'a', Gem::Version.new(0), Gem::Platform.local
+    a   = Gem::NameTuple.new "a", Gem::Version.new(0), Gem::Platform::RUBY
+    a_p = Gem::NameTuple.new "a", Gem::Version.new(0), Gem::Platform.local
 
     assert_equal 1, a_p.<=>(a)
   end
-
 end
-
